package cli;

import kernels.EnergyDensityKernel;
import models.*;

import org.apache.commons.math3.complex.Complex;

/**
 * (For Fig 3b) Computes the lowest eigenvalue of the smeared energy density,
 * for a model with S-matrix consisting of \(n\) pairs of sinh-Gordon factors 
 * with complex coupling constant, \(B=1\pm k a i\), where \(k=1..n\),
 * and \(a>0\) is a constant. \(n\) is varied over a certain range.
 * 
 * Takes four command line arguments: the number \(N\) of discretization steps,
 * the rapidity cutoff \(R\), the maximum number of S-matrix factors pairs (\(n\)
 * is varied from 1 to this value), and the step size in the coupling, \(a\). 
 */
public class ComplexCouplingsAnalyzer extends CLIAnalyzer {

    private static double lowestEv(IntegrableModel model, int coshPower, int n,
            double cutoff) {
        EnergyDensityKernel k = new EnergyDensityKernel(model, 1.0, 0.0,
                coshPower, 0.1);
        double ev = k.findLowestEigenvalue(n, cutoff);
        return ev;
    }

    public static void main(String[] args) {

        int N = Integer.parseInt(args[0]);
        double R = Double.parseDouble(args[1]);
        int couplingsMax = Integer.parseInt(args[2]);
        double a = Double.parseDouble(args[3]);

        String header = "Product model by number of complex-conjugate factors, N="+ N + ", R="+ R
                + ", up to " + couplingsMax + " factors in steps of " + a;
        System.out.println(header);

        double[][] res = new double[couplingsMax][2];

        for (int factors = 1; factors <= couplingsMax; factors++) {

            Object[] b = new Object[factors];
            for (int i = 0; i < factors; i++) {
                b[i] = new Complex(1.0, (i+1) * a);
            }

            IntegrableModel model = new ProductModel(b);

            res[factors - 1][0] = factors;
            res[factors - 1][1] = lowestEv(model, factors, N, R);
        }

        writeDatafile("ComplexCouplings", header, res);
    }
}
