package cli;

import kernels.EnergyDensityKernel;
import models.*;



/**
 * (For Fig 5) Computes the lowest eigenvalue of the smeared "nonstandard" energy density
 * defined with the function \(F_P(\theta) = \big((1-\nu) + \nu \cosh\theta \big) F_\mathrm{min}(\theta+i\pi)\). 
 * This is done for the free and for the sinh-Gordon model (\(B=1)\), with \(\nu\) being varied 
 * between given limits.
 * 
 * Takes four command line arguments: the number \(N\) of discretization steps,
 * the rapidity cutoff \(R\), the maximum value of \(|\nu|\) and the number of steps
 * to divide this interval in \(\nu\) in.
 */
public class NonstandardDensityAnalyzer extends CLIAnalyzer
{   

    public static void main(String[] args) {
        int N = Integer.parseInt(args[0]);
        double R = Double.parseDouble(args[1]);
        double nuMax = Double.parseDouble(args[2]);
        int nuN = Integer.parseInt(args[3]);
        
        String header = "Free / sinh-Gordon model with nonstandard density, N="+N+", R="+R + ", |nu| < " + nuMax + " in  " + nuN + " steps";

        System.out.println(header);

        double[][] res = new double[nuN][3];

        IntegrableModel model1 = new FreeModel();
        IntegrableModel model2 = new SinhGordonModel(1.0);

        for (int i = 0; i < nuN; i++) {
            double nu = -nuMax + 2 * nuMax / (nuN - 1) * i;
            EnergyDensityKernel k1 = new EnergyDensityKernel(model1, 1.0, nu, 0, 0.1);
            if (Math.abs(nu) >= 0.5) {
                k1.disableEVCheck();
            }
            EnergyDensityKernel k2 = new EnergyDensityKernel(model2, 1.0,  nu, 0, 0.1);
            if (Math.abs(nu) >= 0.5 / model2.FminIpi(10.0)) {
                k2.disableEVCheck();
            }

            res[i][0] = nu;
            res[i][1] = k1.findLowestEigenvalue(N, R);
            res[i][2] = k2.findLowestEigenvalue(N, R);
        }

        writeDatafile("NonstandardDensity", header, res);

    }
}
