package cli;

import kernels.EnergyDensityKernel;
import models.*;

/**
 * (For Fig 4) Computes the lowest eigenvalue of the smeared energy density,
 * for the sinh-Gordon model (\(B=1\)), with varying width of the Gaussian
 * "smearing function".
 * 
 * Takes five command line arguments: the number \(N\) of discretization steps,
 * the rapidity cutoff \(R\), the minimum and maximum smearing width \(\sigma\),
 * and the number of steps in which \(\sigma\) traverses this interval.
 */
public class SmearingWidthAnalyzer extends CLIAnalyzer {

    private static double lowestEv(IntegrableModel model, double sigma,
            double alpha, int n, double cutoff) {
        EnergyDensityKernel k = new EnergyDensityKernel(model, 1.0, alpha, 0, sigma);
        double ev = k.findLowestEigenvalue(n, cutoff);
        return ev;
    }

    public static void main(String[] args) {
        int N = Integer.parseInt(args[0]);
        double R = Double.parseDouble(args[1]);
        double smearingMin = Double.parseDouble(args[2]);
        double smearingMax = Double.parseDouble(args[3]);
        int smearingN = Integer.parseInt(args[4]);

        String header = "Sinh-Gordon lowest eigenvalues by smearing width, N="
                + N + ", R=" + R + ", sigma: " + smearingMin + " to "
                + smearingMax + " in " + smearingN + " steps";
        System.out.println(header);

        IntegrableModel sinhModel = new SinhGordonModel(1.0);
        double[][] res = new double[smearingN][2];

        for (int i = 0; i < smearingN; i++) {

            double smearing = smearingMin + (smearingMax - smearingMin) / (smearingN - 1) * i;

            res[i][0] = smearing;
            res[i][1] = lowestEv(sinhModel, smearing, 0.0, N, R);
        }

        writeDatafile("SmearingWidth", header, res);

    }
}
