package kernels;

import models.IntegrableModel;


/**
 * The smeared energy density \(T^{00}(f)\) of an integrable model at one-particle level,
 * considered as an integral kernel operator.
 * 
 * The kernel of this operator is
 * 
 *   \[ K(\theta,\eta) = \frac{\mu^2}{2\pi} \cosh^2 \big(\frac{\theta+\eta}{2}\big) \, 
 *   	    P(\cosh(\theta-\eta)) \, F_\text{min}(\theta-\eta+i\pi) \,
 *   		 \tilde{f} (\mu\cosh\theta-\mu\cosh\eta). \] 
 *
 * Here \(\mu\) is the particle mass, a given number. \(P\) is a polynomial, for which this class supports some options.
 * \( F_\text{min} \) is the minimal solution of the model, cf. {@link models.IntegrableModel}. \(f\) is a real-valued test function
 * of one real variable; this class assumes that it is a Gaussian, namely
 * \[ \tilde f(p) = \exp\big( - p^2 \sigma^2 / \mu^2 \big),\]
 *  where the width \(\sigma\) can be controlled.
 */
public class EnergyDensityKernel extends OneParticleKernel {
    
	/**
	 * The integrable model associated with the energy density.
	 */
    protected IntegrableModel model;
    
    /**
     * The mass of the scalar particle.
     */
    protected double mass;
    
    /**
     * The standard deviation of the Gaussian smearing function
     */
    protected double sigma;
    
    /**
     * The parameter \(\nu\) associated with the polynomial \(P\), see {@link #EnergyDensityKernel(IntegrableModel, double, double, int, double)}.
     */
    protected double nu;
    
    /**
     * The parameter \(n\) associated with the polynomial \(P\), see {@link #EnergyDensityKernel(IntegrableModel, double, double, int, double)}.
     */
    protected int coshPower;

    /**
     * Retrieve the integrable model associated with this kernel
     * @return the integrable model
     */
    public IntegrableModel getModel() {
        return model;
    }

    /**
     * Retrieve the mass \(\mu\) associated with this energy density. 
     * @return the mass
     */
    public double getMass() {
        return mass;
    }
    
    /**
     * Construct an energy density kernel for a specific integrable model.
     * 
     * The polynomial \(P\) here is given by
     * \[  P(x) = \big((1-\nu) + \nu x\big) \big( \frac{x+1}{2} \big)^n  \]
     * where \(n\) and \(\nu\) are passed as parameters.
     * 
     * The smearing function is a Gaussian with standard deviation \(\sigma\).
     *  
     * @param model the integrable model to use
     * @param mass the mass \(\mu\) of the particle
     * @param nu parameter \(\nu\) in the polynomial \(P\), see above
     * @param coshPower parameter \(n\) in the  polynomial \(P\), see above
     * @param sigma standard deviation \(\sigma\) of the smearing function
     */
    public EnergyDensityKernel(IntegrableModel model, double mass, double nu, int coshPower, double sigma) {
        this.model = model;
        this.mass = mass;
        this.sigma = sigma;
        this.nu = nu;
        this.coshPower = coshPower;
    }
    
    /**
     * Return an evaluation of the smearing function, that is, the function \(\tilde f\) in the definition of the kernel.
     * 
     * @param p the argument of the smearing function
     * @return the value of the (Fourier transform of) the smearing function, \(\tilde f (p) \). 
     */
    protected double smearing(double p) {
        return Math.exp(-Math.pow(p/mass*sigma, 2));
    }

    public double kernelValue(double theta, double eta) {
    	
    	double momentumDiff = mass * (Math.cosh(theta)-Math.cosh(eta));
    	double val = smearing(momentumDiff);
    	val *= Math.pow(Math.cosh((theta+eta)/2), 2);
    	val *= (1-nu) + nu * Math.cosh(theta-eta);
    	if (coshPower != 0) {
    		val *= Math.pow( 0.5 + 0.5*Math.cosh(theta-eta), coshPower);
    	}
    	val *= model.FminIpi(theta-eta);
    	val *= mass*mass / 2.0 / Math.PI;
    	return val;
    }

}
