package kernels;

import java.util.Arrays;

import org.apache.commons.math3.linear.BlockRealMatrix;
import org.apache.commons.math3.linear.EigenDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;

/**
 * An integration kernel \( K(\theta,\eta) \) corresponding to an operator on the one-particle space
 * of an integrable theory. 
 * 
 * <p>This class is abstract and doesn't provide the actual kernel function;
 * this is left to subclasses. Rather, this class provides conversion routines
 * from the kernel function to matrix elements of the operator, and
 * for approximately finding its spectral data.</p>
 * 
 * <p>To that end, the following orthonormal system of piecewise constant bumps is used as a basis:
 * \[  \varphi_j (x) = \begin{cases}
 *     h^{-1/2} \quad & \text{if }  -R + j h < x < -R+(j+1)h , \\
 *     0 & \text{otherwise} ,
 *  \end{cases} \]
 *  
 *  with \(j=0,\ldots,N-1\), where \(R>0\) is a given "cutoff", \(N\) the number of basis elements, and \(h=2R/N\).
 *  All approximation results for the eigenvectors, the spectrum, etc. of the operator will depend on these
 *  approximation parameters \(R\) and \(N\). </p>
 * 
 */
public abstract class OneParticleKernel
{
    
	private boolean evCheckEnabled = true;
	
    /**
     * Evaluates the kernel \( K(\theta,\eta) \) at a given position \( \theta,\eta \).
     *  
     * @param theta first argument of the kernel
     * @param eta second argument of the kernel
     * @return value of the kernel at that position.
     */
    public abstract double kernelValue(double theta, double eta);

    /**
     * Disable the plausibility check of decaying eigenfunctions for this kernel.
     * This should be called once, when for this specific kernel it is not expected that the eigenfunctions decay.
     * The check performed by {@link #checkPlausibility} will then be disabled for all method calls. 
     * 
     * @see #findLowestEigenvector
     * @see #findSpectrum
     */
    public void disableEVCheck() {
    	this.evCheckEnabled = false;
    }
    
    /**
     * Retrieve the matrix element of this kernel between two basis vectors. 
     * 
     *  The matrix elements \( (\varphi_i , A \varphi_j) \) are approximated using the midpoint rule.
     *   
     * @param i index of the first basis vector
     * @param j index of the second basis vector
     * @param n number of basis vectors
     * @param cutoff the "cutoff" \(R\)
     * @return the matrix element \( (\varphi_i , A \varphi_j) \)
     */
    protected double matrixElement(int i, int j, int n, double cutoff) {
    	double step = 2.0*cutoff/n;
    	double theta = -cutoff + step*(i + 0.5);
    	double eta = -cutoff + step*(j + 0.5);
        double mtxe = step * kernelValue(theta, eta); 
        return mtxe;
    }
    
    
    /**
     * Compute a matrix approximation of this operator, in the basis described above.
     *   
     * @param n number of basis vectors
     * @param cutoff the "cutoff" \(R\)
     * @return a matrix approximation of this operator in the given basis
     */
    public RealMatrix asRealMatrix(int n, double cutoff) {
        RealMatrix matrix = new BlockRealMatrix(n, n);
        for (int i = 0; i < n; i++) {
            for (int j = 0; j <= i; j++) {
                double mtxe = matrixElement(i, j, n, cutoff);
                matrix.setEntry(i, j, mtxe);
                matrix.setEntry(j, i, mtxe);
            }
        }
        return matrix;            
    }

    /**
     * Compute an approximate eigen-decomposition of this operator, in the basis described above.
     *   
     * @param n number of basis vectors
     * @param cutoff the "cutoff" \(R\)
     * @return eigen-decomposition of this operator in the given basis
     */
    public EigenDecomposition findEigendec(int n, double cutoff) {
        RealMatrix a = this.asRealMatrix(n, cutoff);
        return new EigenDecomposition(a);
    }

    /**
     * Check whether an eigen-decomposition of this operator is plausible, 
     * namely, whether its lowest eigenfunction decays at the end of the interval \([-R,R]\).
     * 
     * This is useful for our main use case, when we want to find the eigenfunction for the lowest eigenvector
     * independent of the cutoff \(R\).
     * The method also checks whether the eigenvalues are all real.
     * (We expect to deal only with symmetric operators.) 
     * 
     * @see #disableEVCheck()
     * @see #findEigendec(int, double)
     *   
     * @param ed the eigen-decomposition to verify
     * @param n number of basis vectors
     * @throws RuntimeException if the matrix has complex eigenvalues
     * @throws RuntimeException if the lowest eigenfunction does not decay
     */
    protected void checkPlausibility(EigenDecomposition ed, int n) {
    	if (ed.hasComplexEigenvalues()) {
    		throw new RuntimeException("complex eigenvalues detected");
    	}
    	if (evCheckEnabled) {
    		RealVector v = ed.getEigenvector(n-1);
    		double eps = 0.0;
    		for (int i = 0; i < 5; i++) {
    			eps += Math.abs(v.getEntry(i));
    			eps += Math.abs(v.getEntry(n-1-i));
    		}
    		if (eps > 1e-4) {
    			throw new RuntimeException("lowest eigenfunction does not decay");
    		}
    	}
    }
    
    /**
     * Approximate the eigenfunction for the lowest eigenvalue of this operator, in the basis described above.
     * 
     * @param n number of basis vectors
     * @param cutoff the "cutoff" \(R\)
     * @return the lowest eigenvector in the given basis
     */
    public RealVector findLowestEigenvector(int n, double cutoff) {
        RealMatrix a = this.asRealMatrix(n, cutoff);
        EigenDecomposition e = new EigenDecomposition(a);
        checkPlausibility(e, n);
        return e.getEigenvector(a.getRowDimension()-1);
    }

    /**
     * Approximate the spectrum of this operator, in the basis described above.
     * The eigenvalues are returned in ascending order. 
     *   
     * @param n number of basis vectors
     * @param cutoff the "cutoff" \(R\)
     * @return the eigenvalues of the operator in this approximation
     */
    public double[] findSpectrum(int n, double cutoff) {
        RealMatrix a = this.asRealMatrix(n, cutoff);
        EigenDecomposition e = new EigenDecomposition(a);
        checkPlausibility(e, n);
        double[] evs =  e.getRealEigenvalues();
        Arrays.sort(evs);
        return evs;
    }

    /**
     * Approximate the lowest eigenvalue of this operator, using the basis described above.
     * 
     * @param n number of basis vectors
     * @param cutoff the "cutoff" \(R\)
     * @return the lowest eigenvalue of the operator in this approximation
     */
    public double findLowestEigenvalue(int n, double cutoff) {
        return findSpectrum(n, cutoff)[0];
    }

}
