package models;

import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Comparator;

import org.apache.commons.math3.analysis.UnivariateFunction;

/**
 * A univariate real function that caches previous evaluations, 
 * so that they can be returned faster if the function is evaluated
 * at the same point several times.
 * 
 * The actual function to be evaluated is passed to the constructor
 * as a <code>UnivariateFunction</code> object   
 */
public class CachedFunction implements UnivariateFunction
{
    private SortedMap<Double,Double> cache;
    private long cacheHits = 0;
    private long evaluations = 0;
    private UnivariateFunction f;

    /**
     * Constructs a cached function, using a given function f for obtaining the actual values of the function.
     * 
     * For the purposes of caching, two arguments of the function will be considered equal if they are closer than
     * the given precision <code>epsilon</code>. That is, the function may return the same value for two arguments
     * that are closed that <code>epsilon</code>.
     * 
     * @param f the function to be cached
     * @param epsilon the precision used for caching, see above
     */
    public CachedFunction(UnivariateFunction f, final double epsilon) {
    	
    	this.f = f;
    	
    	Comparator<Double> epsComp = new Comparator<Double>(){
			public int compare(Double d1, Double d2) {
				if (Math.abs(d1-d2) < epsilon) {
					return 0;
				} else if (d1 > d2) {
					return 1;
				} else {
					return -1;
				}
			}    		    		
    	};

    	cache = new TreeMap<Double,Double>(epsComp);
    }

    
    public double value(double x) {
    	Double cached = cache.get(x);
    	if (cached != null) {
    		cacheHits++;
    		return cached;
    	} else {    		
        	evaluations++;
    		double val = f.value(x);
    		cache.put(x, val);
            return val;
    	}    	
    }

    /**
     * Retrieve the number of cache hits, that is, the number of times that an evaluation of the function was actually
     * done using the cache. This is for testing purposes only.
     * 
     * @return the number of cache hits
     */
    public long getCacheHits() {
    	return cacheHits;
    }

}
