package models;

import org.apache.commons.math3.analysis.UnivariateFunction;
import org.apache.commons.math3.complex.Complex;

/**
 * An integrable model with scattering function composed of the product of several sinh-Gordon scattering functions.
 * 
 * These sinh-Gordon type factors can have different coupling constants, and the coupling constant can be complex,
 * as long as the complex values appear in complex-conjugate pairs. An overall minus sign in the scattering function,
 * like in the Ising model, can also be included. 
 * 
 */
public class ProductModel extends IntegrableModel
{
    private CachedFunction fminIpi;
    private int coshPower;
        
    /**
     * Constructs a new product model corresponding to a given list of sinh-Gordon coupling constants,
     * but no overall minus sign. Identical to {@link #ProductModel(Object[], boolean)} with the last parameter set to <code>false</code>.
     */
    public ProductModel(final Object[] coupling) {
    	this(coupling, false);
    }
    
    /**
     * Constructs a new product model corresponding to a given list of sinh-Gordon coupling constants.
     * 
     * These constants can be either real or complex; in the latter case, two complex-conjugate factors are added to the scattering function.
     * An overall plus/minus sign can also be included.
     * The overall scattering function is
     * \[ S(\zeta) = \pm \Big( \prod_{j =1}^n \dfrac{\sinh \zeta - i \sin B_j \pi/2 }{\sinh \zeta + i \sin B_j \pi/2 }\Big)
     *    \Big( \prod_{j =1}^n \dfrac{\sinh \zeta - i \sin C_j \pi/2 }{\sinh \zeta + i \sin C_j \pi/2 }
     *           \dfrac{\sinh \zeta - i \sin \bar C_j \pi/2 }{\sinh \zeta + i \sin \bar C_j \pi/2 } \Big)
     *  \] 
     * where \(B_j\) are the real and \(C_j\) are the complex coupling constants. The real part of \(B_j,C_j\) is expected 
     * to be in the interval \( [0,2] \).  
     * 
     * Technically, these coupling constants are passed as an array that should contain either <code>Double</code> objects or
     * <code>org.apache.commons.math3.complex.Complex</code> objects, or a combination of both. The plus/minus sign is determined
     * by the parameter <code>hasMinus</code>
     * 
     * @param coupling the list of coupling constants, as described above
     * @param hasMinus whether an overall minus sign is included in the S function.
     */
    public ProductModel(final Object[] coupling, final boolean hasMinus) {
    	
    	int nCoupling = 0;
    	for (Object o: coupling) {
    		if (o instanceof Complex) {
    			nCoupling += 2;
    		} else {
    			nCoupling += 1;
    		}
    	}
    	
    	if (hasMinus) {
    		this.coshPower =  1 - 2 * ((nCoupling+1) / 2);
 
    	} else {
    		this.coshPower = -2 * (nCoupling / 2);
    	}
    	        
        UnivariateFunction fminI = new UnivariateFunction() {
            public double value(double theta) {
            	double prod = 1.0;
            	for (Object b : coupling) {
            		if (b instanceof Double) {
            			prod *= SinhTools.sinhIntegral(theta, (Double) b);
            		} else if (b instanceof Complex) {
            			prod *= SinhTools.sinhIntegralCCPair(theta, (Complex) b);
            		} else {
            			throw new IllegalArgumentException("coupling neither Double nor Complex: "+b);
            		}
            	}
                return prod;
            }
        };
       
        fminIpi = new CachedFunction(fminI, 1e-14);
    }
    
    @Override
    public double FminIpi(double theta) {
        double val = fminIpi.value(Math.abs(theta));
   		val *= Math.pow(Math.cosh(theta/2.0), this.coshPower);
    	return val;
    }    
    
}
