package models;

import static org.junit.Assert.*;

import org.junit.Test;


/**
 * Unit tests for the ProductModel class.
 * 
 * This compares the consistency of the {@link ProductModel} implementation with special cases
 * that are implemented in other classes as well, for example the classes
 * {@link IsingModel} and {@link SinhGordonModel}. 
 *
 */
public class ProductModelTest {

	private void compareModels(String msg, IntegrableModel model1, IntegrableModel model2) {
		compareModels(msg, new IntegrableModel[] {model1}, 0, model2 );
	}
	
	private void compareModels(String msg, IntegrableModel[] expModels, int coshPower, IntegrableModel model2) {
		
		for (double t = 0.0; t < 10.1; t++) {
			double expected = 1.0;
			for (IntegrableModel mod : expModels) {
				expected *= mod.FminIpi(t);
			}
			if (coshPower != 0) {
				expected *= Math.pow(Math.cosh(t/2), coshPower);
			}
			double actual = model2.FminIpi(t);
			assertEquals(msg + ", t="+t, expected, actual, 1e-3);
		}
		
	}
	
	
	/**
	 * Verifies the product model against the sinh-Gordon model with various couplings. 
	 */
	@Test
	public void comparisonWithSinhGordon() {
		
		compareModels("Comparing product model with Sinh b=1", 
				new SinhGordonModel(1.0),
				new ProductModel( new Object[]{ new Double(1.0)} )
		);
		
		compareModels("Comparing product model with Sinh b=0.5", 
				new SinhGordonModel(0.5),
				new ProductModel( new Object[]{ new Double(0.5)} )
		);
		
	}

	/**
	 * Verifies the product model against the Ising model. 
	 */
	@Test
	public void comparisonWithIsing() {
		
		compareModels("Comparing product model with Ising", 
				new IsingModel(),
				new ProductModel( new Object[]{}, true )
		);
				
	}

	/**
	 * Verifies the product model against the sinh-Gordon model with an additional minus sign ("sinh-Ising model"). 
	 */
	@Test
	public void comparisonWithSinhIsing() {
		
		compareModels("Comparing product model with Sinh-Ising b=1", 
				new SinhIsingModel(1.0),
				new ProductModel( new Object[]{ 1.0 }, true )
		);
		
		compareModels("Comparing product model with Sinh-Ising b=0.5", 
				new SinhIsingModel(0.5),
				new ProductModel( new Object[]{ 0.5 }, true )
		);
		
	}

	/**
	 * Verifies the product model against a manually computed product of S-matrices from three sinh-Gordon models.
	 *  
	 * Note that the minimal solution of the product model is obtained by the product of the minimal solutions 
	 * in the sinh-Gordon models, with an additional factor of \( \big(\cosh (\zeta/2)\big)^{-2} \). 
	 */
	@Test
	public void comparisonWithSinhProduct() {
		IntegrableModel[] sinhM = { new SinhGordonModel(1.0),
								   new SinhGordonModel(0.7),
								   new SinhGordonModel(0.3) };
		
		IntegrableModel product = new ProductModel( new Object[] {1.0, 0.7, 0.3} );
		
		compareModels("Comparing product of three sinh-Gordons", sinhM, -2, product);		
	}

}
