package models;

import static java.lang.Math.*;

import org.apache.commons.math3.analysis.UnivariateFunction;
import org.apache.commons.math3.analysis.integration.SimpsonIntegrator;
import org.apache.commons.math3.analysis.integration.UnivariateIntegrator;
import org.apache.commons.math3.complex.Complex;

/**
 * This class provides helper functionality for computing the minimal solutions
 * of various models, given by the exponential of a certain integral expression.
 * 
 */
public class SinhTools
{
    /**
     * Computes an integral expression that occurs in the minimal solution of sinh-Gordon-type models.
     * Explicitly, we compute
     * 
     *   \[ \exp 8\int_{0}^{\infty} \frac{dx}{x}\frac{\sinh\Big(\frac{x B}{4}\Big)\sinh\Big(\frac{x}{2}(1-\frac{B}{2}) \Big)
     *   \sinh\frac{x}{2}}{\sinh^2 x} \sin^2\Big( \frac{x\theta}{2\pi}\Big) \]
     *  
     *  where \(\theta \in \mathbb{R} \) and \( B \in (0,2) \) are given as a parameter.  
     *  
     *  The integral is numerically approximated using the composite Simpson's rule.
     *  
     * @param theta the parameter \( \theta \) in the integrand
     * @param b the parameter \( B \) in the integrand
     * @return the value of the integral expression as described above
     */
    public static double sinhIntegral(final double theta, final double b) {

        UnivariateIntegrator integrator = new SimpsonIntegrator(1e-12, 1e-15, 
        		SimpsonIntegrator.DEFAULT_MIN_ITERATIONS_COUNT, SimpsonIntegrator.SIMPSON_MAX_ITERATIONS_COUNT);
        
        UnivariateFunction integrand = new UnivariateFunction() {
                public double value(double x) {
                    return 1.0/x*sinh(x*b/4)*sinh((2-b)/4*x)*sinh(x/2)/pow(sinh(x), 2) * pow(sin(x*theta/2/PI), 2);
                }
            };
        double intval = integrator.integrate(1000000, integrand, 1e-10, 30);
        return exp(8*intval);
    }

    /**
     * Computes the limit of the integral expression {@link #sinhIntegral} as \(\theta \to \infty\).
     * By the Riemann-Lebesgue lemma, this limit is given by
     * 
     *   \[ \exp 4\int_{0}^{\infty} \frac{dx}{x}\frac{\sinh\Big(\frac{x B}{4}\Big)\sinh\Big(\frac{x}{2}(1-\frac{B}{2}) \Big)
     *   \sinh\frac{x}{2}}{\sinh^2 x}  \]
     *  
     *  The integral is numerically approximated using the composite Simpson's rule.
     *  
     * @param b the parameter \( B \) in the integrand
     * @return the value of the integral expression as described above
     */
    public static double sinhIntegralLimit(final double b) {

        UnivariateIntegrator integrator = new SimpsonIntegrator(1e-12, 1e-15,
        		SimpsonIntegrator.DEFAULT_MIN_ITERATIONS_COUNT, SimpsonIntegrator.SIMPSON_MAX_ITERATIONS_COUNT);
        
        UnivariateFunction integrand = new UnivariateFunction() {
                public double value(double x) {
                    return 1.0/x*sinh(x*b/4)*sinh((2-b)/4*x)*sinh(x/2)/pow(sinh(x), 2);
                }
            };
        double intval = integrator.integrate(1000000, integrand, 1e-10, 30);
        return exp(4*intval);
    }

    /**
     * Computes the product of two integral expression as in {@link #sinhIntegral} but for a complex value \(B\) 
     * and for its complex conjugate.
     * 
     * The expression is explicitly given by 
     *   \[ \exp 8\int_{0}^{\infty} \frac{dx}{x}\frac{ 2 \operatorname{Re} \Big(\sinh\Big(\frac{x B}{4}\Big)\sinh\Big(\frac{x}{2}(1-\frac{B}{2})  \Big) \Big)
     *   \sinh\frac{x}{2}}{\sinh^2 x} \; \sin^2\Big( \frac{x\theta}{2\pi}\Big) \]
     *  
     *  where \(\theta \in \mathbb{R} \) and \( B \in (0,2) + i \mathbb{R} \) are given as parameters.  
     * 
     * The integral is numerically approximated using the composite Simpson's rule.
     * @param theta the parameter \( \theta \) in the integrand
     * @param b the complex parameter \( B \) in the integrand
     * @return the value of the integral expression as described above
     */ 
    public static double sinhIntegralCCPair(final double theta, final Complex b) {

        UnivariateIntegrator integrator = new SimpsonIntegrator(1e-12, 1e-15, 
        		SimpsonIntegrator.DEFAULT_MIN_ITERATIONS_COUNT, SimpsonIntegrator.SIMPSON_MAX_ITERATIONS_COUNT);

        UnivariateFunction integrand = new UnivariateFunction() {
                public double value(double x) {
                	Complex sinhFact1 = b.multiply(x/4).sinh();
                	Complex sinhFact2 = new Complex(2, 0).subtract(b).multiply(x/4).sinh();
                	double sinhFact = sinhFact1.multiply(sinhFact2).getReal()*2;
                    return 1.0/x*sinhFact*sinh(x/2)/pow(sinh(x), 2) * pow(sin(x*theta/2/PI), 2);
                }
            };
        double intval = integrator.integrate(1000000, integrand, 1e-10, 30);
        return exp(8*intval);
    }

    /**
     * Computes the limit of the integral expression {@link #sinhIntegralCCPair} as \(\theta \to \infty\).
     * By the Riemann-Lebesgue lemma, this limit is given by
     *  \[ \exp 4\int_{0}^{\infty} \frac{dx}{x}\frac{ 2 \operatorname{Re} \Big(\sinh\Big(\frac{x B}{4}\Big)\sinh\Big(\frac{x}{2}(1-\frac{B}{2})  \Big) \Big)
     *     \sinh\frac{x}{2}}{\sinh^2 x}  \]
     *  
     *  
     *  The integral is numerically approximated using the composite Simpson's rule.
     *  
     * @param b the parameter \( B \) in the integrand
     * @return the value of the integral expression as described above
     */
    public static double sinhIntegralCCPairLimit(final Complex b) {

        UnivariateIntegrator integrator = new SimpsonIntegrator(1e-12, 1e-15, 
        		SimpsonIntegrator.DEFAULT_MIN_ITERATIONS_COUNT, SimpsonIntegrator.SIMPSON_MAX_ITERATIONS_COUNT);

        UnivariateFunction integrand = new UnivariateFunction() {
                public double value(double x) {
                	Complex sinhFact1 = b.multiply(x/4).sinh();
                	Complex sinhFact2 = new Complex(2, 0).subtract(b).multiply(x/4).sinh();
                	double sinhFact = sinhFact1.multiply(sinhFact2).getReal()*2;
                    return 1.0/x*sinhFact*sinh(x/2)/pow(sinh(x), 2) ;
                }
            };
        double intval = integrator.integrate(1000000, integrand, 1e-10, 30);
        return exp(4*intval);
    }

}
