package models;

import static org.junit.Assert.*;

import org.apache.commons.math3.complex.Complex;
import org.junit.Test;


/**
 * Unit tests for the {@link SinhTools} class.
 *
 */
public class SinhToolsTest {

	
	/**
	 * Comparison of the function sinhIntegral to some values computed externally
	 * with Maple to higher floating point precision (tolerance 1e-20).
	 */
	@Test
	public void testSinhIntegral() {
		
		double prec = 1e-12;
		
		assertEquals(1.03157901353779514476969801058,
				SinhTools.sinhIntegral(1.0, 1.0), prec);

		assertEquals(1.10201983196157370929728224428,
				SinhTools.sinhIntegral(2.0, 1.0), prec);

		assertEquals(1.07836648136920973176139239832,
				SinhTools.sinhIntegral(2.0, 0.5), prec);

		assertEquals(1.19672220495232805797153069762,
				SinhTools.sinhIntegral(10.0, 0.5), prec);

	}

	/** 
	 * Comparison of {@link models.SinhTools#sinhIntegral} for large values of theta
	 * with its limit computed in {@link models.SinhTools#sinhIntegralLimit}.
	 */
	@Test
	public void testSinhIntegralLimit() {
		double finiteInt = SinhTools.sinhIntegral(50, 1.0);
		double limitInt =  SinhTools.sinhIntegralLimit(1.0);
		String msg = "Testing limit of sinh Integral";
				
		assertEquals(msg, finiteInt, limitInt, 1e-10);				
	}

	/** 
	 * Comparison of {@link models.SinhTools#sinhIntegralCCPair} for large values of theta
	 * with its limit computed in {@link models.SinhTools#sinhIntegralCCPairLimit}.
	 */
	@Test
	public void testSinhIntegralCCLimit() {
		Complex c = new Complex(1.0, 0.5);
		double finiteInt = SinhTools.sinhIntegralCCPair(50, c);
		double limitInt =  SinhTools.sinhIntegralCCPairLimit(c);
		String msg = "Testing limit of sinh Integral with complex coupling";
				
		assertEquals(msg, finiteInt, limitInt, 1e-10);				
	}

	/**
	 * Comparison of the function {@link models.SinhTools#sinhIntegralCCPair} to some values computed externally
	 * with Maple at higher floating point precision (tolerance 1e-20).
	 */
	@Test
	public void testSinhIntegralCCPair() {
		double prec = 1e-12;

		assertEquals(1.11148959840450668619683620201,
				SinhTools.sinhIntegralCCPair(1.0, new Complex(1,1)), prec);

		assertEquals(1.26449797806599613970533667556,
				SinhTools.sinhIntegralCCPair(2.0, new Complex(1,0.5)), prec);
		

	}

	private void testCCPairOneRealValue(double t, double b) {
		double singleInt = SinhTools.sinhIntegral(t, b);
		double doubleInt = SinhTools.sinhIntegralCCPair(t, new Complex(b,0));
		String msg = "Testing t="+t+", b="+b;
				
		assertEquals(msg, singleInt*singleInt, doubleInt, 1e-10);				
	}	

	/**
	 * Comparison of the output of {@link models.SinhTools#sinhIntegralCCPair} to {@link models.SinhTools#sinhIntegral}
	 * in the case where the imaginary part of the coupling is zero.
	 */
	@Test
	public void testSinhIntegralCCPairReal() {
		for (double t = 0.0; t <= 10.0; t += 1.0) {
			for (double b = 0.1; b <= 1.9; b += 0.1) {
				testCCPairOneRealValue(t, b);
			}
		}
	}

}
